/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Toporov Konstantin.
 */

package lookout;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import lookout.settings.ProgramSettings;

import java.util.Collection;
import java.awt.BorderLayout;

import java.awt.Dimension;

import java.net.URL;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.event.KeyEvent;

import products.ProdGroup;
import products.ProductInBase;
import java.text.DecimalFormat;
import java.awt.GridLayout;

import javax.swing.filechooser.FileNameExtensionFilter;
import java.io.*;

import java.util.Vector;
import javax.swing.*;
import java.util.zip.*;


public class ExportDialog  extends JDialog implements ActionListener{
    private static int WIDTH_D;
    private static int HEIGHT_D;
    private static final String MOVE_GR_UP = "moveGroupUp";
    private static final String MOVE_GR_DOWN = "moveGroupDown";
    private static final String DELETE_GR = "deleteGroup";
    private static final String START_EXPORT = "begin export";
    private static final String CANCEL = "quit dialog";
    
    private ProgramSettings settings;

    private JTextArea text;
    private String hat;
    private String body;
    private Collection groups;
    private Collection products;
    private JList grList;
    private JRadioButton fooddb;
    private JRadioButton diabetjar;
    
    public ExportDialog(JFrame owner,Collection groups, Collection products){
        super(owner);

        this.groups = groups;
        this.products = products;
        settings = ProgramSettings.getInstance();
        setTitle("Экспорт базы данных в файл food.db/diabet.jar");
        
        if (settings.getIn().getSize()==4){
            WIDTH_D = 600;
            HEIGHT_D = 430;
        }
        else{
            WIDTH_D = 550;
            HEIGHT_D = 390;
        }
        
        JPanel left = new JPanel(new BorderLayout());
        grList = new JList(groups.toArray());
        grList.setFont(settings.getIn().getFont(grList.getFont()));
        grList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        
        JScrollPane listScroller = new JScrollPane(grList);
        listScroller.setPreferredSize(new Dimension(200,0));
        
        
        
        JToolBar bar = new JToolBar();
        bar.add(makeButton("Up","Up",MOVE_GR_UP,
                settings.getIn().StrSize("Переместить группу вверх")));
        bar.add(makeButton("Down","Dn",MOVE_GR_DOWN,
                settings.getIn().StrSize("Переместить группу вниз")));
        bar.addSeparator();
        bar.add(makeButton("Delete","Dl",DELETE_GR,
                settings.getIn().StrSize("Удалить группу")));
        bar.setFloatable(false);
        
        left.add(listScroller, BorderLayout.CENTER);
        
        JPanel right = new JPanel(new BorderLayout());
        
        text = new JTextArea();
        text.setEditable(false);
        text.setFont(settings.getIn().getFont(text.getFont()));
        text.setText(getHat()+getBody());
        right.add(new JScrollPane(text), BorderLayout.CENTER);

        fooddb = new JRadioButton(settings.getIn().StrSize("Экспортировать в файл food.db"));
        diabetjar = new JRadioButton(settings.getIn().StrSize("Заменить данные в diabet.jar"));
        fooddb.setSelected(true);
        ButtonGroup bgr = new ButtonGroup();
        bgr.add(fooddb);
        bgr.add(diabetjar);
        Box box = new Box(BoxLayout.PAGE_AXIS);
        box.add(fooddb);
        box.add(diabetjar);

        JPanel centerPane = new JPanel(new BorderLayout());
        centerPane.add(left,BorderLayout.WEST);
        centerPane.add(right,BorderLayout.CENTER);
        centerPane.add(box,BorderLayout.SOUTH);
        
        JPanel bottom = new JPanel(new GridLayout(1,2));
        JButton startExportBtn = new JButton(settings.getIn().StrSize("Начать экспорт"));
        startExportBtn.addActionListener(this);
        startExportBtn.setActionCommand(START_EXPORT);
        
        JButton cancelBtn = new JButton(settings.getIn().StrSize("Выход"));
        cancelBtn.addActionListener(this);
        cancelBtn.setActionCommand(CANCEL);
        
        bottom.add(startExportBtn);
        bottom.add(cancelBtn);
        getContentPane().setLayout(new BorderLayout());
        
        getContentPane().add(bar, BorderLayout.NORTH);
        getContentPane().add(centerPane, BorderLayout.CENTER);
        getContentPane().add(bottom, BorderLayout.SOUTH);
        
        
        setSize(WIDTH_D,HEIGHT_D);
        //pack();
        setResizable(false);
        setLocationRelativeTo(owner);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        this.getRootPane().registerKeyboardAction(new ActionListener() {
            @Override
                public void actionPerformed(ActionEvent actionEvent) {
                    setVisible(false);
                }
                }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                    JComponent.WHEN_IN_FOCUSED_WINDOW);
    }
    
    private JButton makeButton(String imageName, String altText, 
            String actionCommand, String toolTipText) {
        //Look for the image.
        String filesize;
        if (settings.getIn().getSize()==4) filesize = "24";
        else filesize = "16";
        String imgLocation =  "bttns/" + imageName + filesize+".png";
        URL imageURL = MainFrame.class.getResource(imgLocation);
       
        //Create and initialize the button.
        JButton button = new JButton();
        button.setToolTipText(toolTipText);
        button.setActionCommand(actionCommand);
        button.addActionListener(this);
        
        if (imageURL != null) {                      //image found
            button.setIcon(new ImageIcon(imageURL, altText));
        } else {                                     //no image found
            button.setText(altText);
            System.err.println("Resource not found: "
                               + imgLocation);
        }

        return button;
    }
    
    @Override
   public void actionPerformed(ActionEvent e){
        String cmd = e.getActionCommand();
        if (CANCEL.equals(cmd)){
            this.setVisible(false);
        }
        else if (START_EXPORT.equals(cmd)){
            //тут надо запустить диалог записи файла
            startExport();
        }
        else if (DELETE_GR.equals(cmd)){
            deleteGroup();
        }
        else if (MOVE_GR_UP.equals(cmd)){
            moveGrUp();
        }
        else if (MOVE_GR_DOWN.equals(cmd)){
            moveGrDown();
        }
   }
   
   private String getHat(){
       hat = "<MENU>\n";
       
       if (groups.size()>0)
       for (Object item:groups){
           hat += ((ProdGroup)item).getName() + "\t" + 
                   (((ProdGroup)item).getId()+1) +  "\n";
       }
       return hat;
   }
   private String getBody(){
       DecimalFormat frt = new DecimalFormat("0.0");
       body = "<DATA>\n";
       if (products.size()>0)
       for (Object item:products){
           if (((ProductInBase)item).getName().trim().length()>2){
            body += ((ProductInBase)item).getName() + "\t" +
                   (frt.format(((ProductInBase)item).getProt()) + "\t" + 
                   frt.format(((ProductInBase)item).getFat())  + "\t" + 
                   frt.format(((ProductInBase)item).getCarb())).replace(',','.')  + "\t" + 
                   ((ProductInBase)item).getGi()  + "\t" + 
                   (((ProductInBase)item).getOwner()+1) + "\n";
           }
       }
       
       return body;
   }
   
   private void startExport(){
       JFileChooser fileSelector = new JFileChooser();
       fileSelector.setDialogTitle("Экспорт базы продуктов");
       FileNameExtensionFilter filter;
       File fl;
       if (fooddb.isSelected()){
           filter = new FileNameExtensionFilter(
                            "Файл базы данных: food.db", "db");
           fl = new File("food.db");
       }
       else{
           filter = new FileNameExtensionFilter(
                            "Программа для мобильного телефона: diabet.jar", "jar");
           fl = new File("diabet.jar");
       }

       fileSelector.setMultiSelectionEnabled(false);
       fileSelector.setFileFilter(filter);
       
       fileSelector.setSelectedFile(fl);
       fileSelector.setAcceptAllFileFilterUsed(false);
       int returnVal = fileSelector.showSaveDialog(this);
       if(returnVal == JFileChooser.APPROVE_OPTION) {
            try{
                File file2write;
                File tmpfile = null;
                if (fooddb.isSelected()){
                    file2write = fileSelector.getSelectedFile();
                }
                else{
                    file2write = new File("food.db");
                    if (file2write.exists()){
                        tmpfile = File.createTempFile(file2write.getName(), null, 
                                new File(System.getProperty("user.dir")));
                        fileCopy(file2write,tmpfile);
                    }
                }
                BufferedWriter out = new BufferedWriter(new OutputStreamWriter
                                 (new FileOutputStream(
                                 file2write
                                 ),"UTF8"));
 
                out.write(hat);
                out.write(body);
                out.close();
                if (diabetjar.isSelected()){
                    String manifest = addFilesToExistingZip(fileSelector.getSelectedFile(),
                        file2write);
                    file2write.delete();
                    if (manifest.length()>0){
                    String jadname = fileSelector.getSelectedFile().getAbsolutePath();
                    jadname = jadname.substring(0, jadname.length()-3) + "jad";
                    
                    BufferedWriter jad = new BufferedWriter(new OutputStreamWriter
                                 (new FileOutputStream(
                                 new File(
                                 jadname
                                 )
                                 ),"UTF8"));
                       jad.write(manifest);
                       jad.newLine();
                       jad.write("MIDlet-Jar-Size: "+fileSelector.getSelectedFile().length());
                       jad.newLine();
                       jad.write("MIDlet-Jar-URL: "+fileSelector.getSelectedFile().getName());
                       jad.newLine();
                       jad.close();
                    }
                }
                if (tmpfile!=null){
                    fileCopy(tmpfile,file2write);
                    tmpfile.delete();
                }
                JOptionPane.showMessageDialog(this, "Экспорт удачно завершен");
            }
            catch(UnsupportedEncodingException ue){
                ue.printStackTrace();
            }
            catch(IOException e){
                e.printStackTrace();
            }
 
    }
                         
   }
   private void fileCopy(File source, File target){
    try{
        FileReader in = new FileReader(source);

        FileWriter out = new FileWriter(target);
        int c;

        while ((c = in.read()) != -1)
            out.write(c);

        in.close();
        out.close();
    }
    catch (IOException e){
        System.err.println(e.getLocalizedMessage());
    }
   }
   private void deleteGroup(){
        if (!grList.isSelectionEmpty()){
            ProdGroup gr = (ProdGroup)grList.getModel().getElementAt(grList.getSelectedIndex());
            
            Vector vc = new Vector(products);
            for (Object item:vc){
                if (gr.getId()==  ((ProductInBase)item).getOwner()){
                    products.remove(item);
                } 
            }
            
          
            groups.remove(gr);
            grList.setListData(groups.toArray());
            text.setText(getHat()+getBody());
        }
   }
   
   private void moveGrUp(){
       if (!grList.isSelectionEmpty() && grList.getSelectedIndex()>0){
           int sel = grList.getSelectedIndex();
           ProdGroup grLo = new ProdGroup((ProdGroup)grList.getModel().getElementAt(sel));
           //grList.setSelectedIndex(sel-1);
           ProdGroup grHi = new ProdGroup((ProdGroup)grList.getModel().getElementAt(sel-1));
           
           Vector vc = new Vector(groups);
           ProdGroup gr  = (ProdGroup)vc.get(sel);//Выбрали нижнюю группу
           gr.setId(grHi.getId());
           gr.setName(grHi.getName());
           
           gr  = (ProdGroup)vc.get(sel-1);//Выбрали нижнюю группу
           gr.setId(grLo.getId());
           gr.setName(grLo.getName());
           
           groups = vc;
           grList.setListData(groups.toArray());
           text.setText(getHat()+body);        
       }
   }
   
   private void moveGrDown(){
       if (!grList.isSelectionEmpty() && grList.getSelectedIndex()<(groups.size()-1) ){
           int sel = grList.getSelectedIndex();
           ProdGroup grHi = new ProdGroup((ProdGroup)grList.getModel().getElementAt(sel));
           //grList.setSelectedIndex(sel-1);
           ProdGroup grLo = new ProdGroup((ProdGroup)grList.getModel().getElementAt(sel+1));
           
           Vector vc = new Vector(groups);
           ProdGroup gr  = (ProdGroup)vc.get(sel+1);//Выбрали нижнюю группу
           gr.setId(grHi.getId());
           gr.setName(grHi.getName());
           
           gr  = (ProdGroup)vc.get(sel);//Выбрали нижнюю группу
           gr.setId(grLo.getId());
           gr.setName(grLo.getName());
           
           groups = vc;
           grList.setListData(groups.toArray());
           text.setText(getHat()+body);        
       }
   }
   public String addFilesToExistingZip(File zipFile,
			 File file) throws IOException {
       String manifest = "";
                // get a temp file
		File tempFile = File.createTempFile(zipFile.getName(), null,
                        new File (System.getProperty("user.dir")));

        // delete it, otherwise you cannot rename your existing zip to it.
		tempFile.delete();

		boolean renameOk=zipFile.renameTo(tempFile);
		if (!renameOk)
		{
                    throw new RuntimeException("could not rename the file "+zipFile.getAbsolutePath()+" to "+tempFile.getAbsolutePath());
		}
                
		byte[] buf = new byte[1024];

		ZipInputStream zin = new ZipInputStream(new FileInputStream(tempFile));
		ZipOutputStream out = new ZipOutputStream(new FileOutputStream(zipFile));
                
		ZipEntry entry = zin.getNextEntry();
		while (entry != null) {
			String name = entry.getName();
                        
                    boolean notInFiles = true;
		    if (file.getName().equals(name)) {
			   notInFiles = false;
			}
		    
			if (notInFiles) {
				// Add ZIP entry to output stream.
				out.putNextEntry(new ZipEntry(name));
				// Transfer bytes from the ZIP file to the output file
				int len;
                                String stbuf;
				while ((len = zin.read(buf)) > 0) {
                                    if (name.contains("MANIFEST.MF")){
                                        stbuf = new String(buf,0,len);
                                        manifest += stbuf;
                                    }
					out.write(buf, 0, len);
				}
			}
			entry = zin.getNextEntry();
		}
		// Close the streams
		zin.close();
		// Compress the files
		//for (int i = 0; i < files.length; i++) {
			InputStream in = new FileInputStream(file);
			// Add ZIP entry to output stream.
			out.putNextEntry(new ZipEntry(file.getName()));
			// Transfer bytes from the file to the ZIP file
			int len;
			while ((len = in.read(buf)) > 0) {
				out.write(buf, 0, len);
			}
			// Complete the entry
			out.closeEntry();
			in.close();
		//}
		// Complete the ZIP file
		out.close();
		tempFile.delete();
                
                return manifest;
	}

  
}
